;------------------------------------------------------------------------------
; This file is part of the ZX Spectrum game 
;       "Husband Chores" by Sebastian Mihai, 2016, http://sebastianmihai.com
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; Manages events such as trash bin becoming full, toilets clogging up, etc.
;
;------------------------------------------------------------------------------

	; these are used when ANDing with current frame count to determine
	; whether to take certain actions during this frame
	; (lower values make for more frequent actions)
	; they can only be set to one less a power of 2 (3, 7, 15, etc.)
	
	CYCLE_WARNING_GRAPHIC_AND_VALUE equ 3	; how often we cycle the warning
											; sign animation
	
	GENERATE_EVENT_FRAME_AND_VALUE equ 31	; how often we roll the die to 
											; see if we'll generate an event
	
	EVENT_GENERATE_CHANCE equ 20			; the die we roll to see if we're
											; generating an event
											; out of 255

	FORCE_EVENT_GENERATION_FRAME_COUNT equ 250	; 10 seconds
											
	; in 8x8 character coordinates
	TRASH_SECOND_FLOOR_X equ 17
	TRASH_SECOND_FLOOR_X_PIXEL equ TRASH_SECOND_FLOOR_X * 8
	TRASH_SECOND_FLOOR_Y equ 17
	TRASH_SECOND_FLOOR_Y_PIXEL equ TRASH_SECOND_FLOOR_Y * 8
	
	TRASH_THIRD_FLOOR_X equ 8
	TRASH_THIRD_FLOOR_X_PIXEL equ TRASH_THIRD_FLOOR_X * 8
	TRASH_THIRD_FLOOR_Y equ 14
	TRASH_THIRD_FLOOR_Y_PIXEL equ TRASH_THIRD_FLOOR_Y * 8
	
	TOILET_SECOND_FLOOR_X equ 14
	TOILET_SECOND_FLOOR_X_PIXEL equ TOILET_SECOND_FLOOR_X * 8
	TOILET_SECOND_FLOOR_Y equ 17
	TOILET_SECOND_FLOOR_Y_PIXEL equ TOILET_SECOND_FLOOR_Y * 8
	
	TOILET_THIRD_FLOOR_X equ 23
	TOILET_THIRD_FLOOR_X_PIXEL equ TOILET_THIRD_FLOOR_X * 8
	TOILET_THIRD_FLOOR_Y equ 14
	TOILET_THIRD_FLOOR_Y_PIXEL equ TOILET_THIRD_FLOOR_Y * 8
	
	TOILET_FOURTH_FLOOR_X equ 12
	TOILET_FOURTH_FLOOR_X_PIXEL equ TOILET_FOURTH_FLOOR_X * 8
	TOILET_FOURTH_FLOOR_Y equ 11
	TOILET_FOURTH_FLOOR_Y_PIXEL equ TOILET_FOURTH_FLOOR_Y * 8
	
	; we'll flash the warning sign red when snow is present, to indicate
	; that it cannot be picked up before the snow is removed
	ACTIVE_ATTRIBUTES1_T equ FLASH_OFF | BRIGHT_ON | PAPER_BLACK | INK_RED
	ACTIVE_ATTRIBUTES1 equ FLASH_OFF | BRIGHT_ON | PAPER_BLACK | INK_YELLOW
	ACTIVE_ATTRIBUTES2 equ FLASH_OFF | BRIGHT_ON | PAPER_BLACK | INK_BLACK
	INACTIVE_ATTRIBUTES equ FLASH_OFF | BRIGHT_ON | PAPER_BLACK | INK_BLACK

	currentAttributes db 0
	eventFrameCounter db 0
	framesSinceLastEvent db 0

	secondFloorToiletClogged db 0
	thirdFloorToiletClogged db 0
	fourthFloorToiletClogged db 0
	
	secondFloorTrashFull db 0
	thirdFloorTrashFull db 0
	
initialize_events:
	ld hl, secondFloorToiletClogged
	ld (hl), 0
	ld hl, thirdFloorToiletClogged
	ld (hl), 0
	ld hl, fourthFloorToiletClogged
	ld (hl), 0
	
	ld hl, secondFloorTrashFull
	ld (hl), 0
	ld hl, thirdFloorTrashFull
	ld (hl), 0
	
	ld hl, framesSinceLastEvent
	ld (hl), 0
	ld hl, eventFrameCounter
	ld (hl), 0
	ld hl, currentAttributes
	ld (hl), ACTIVE_ATTRIBUTES1
	ret
	
	
; Called when the player has pressed the action button, it checks 
; whether the player collides with an event, handling that case, if necessary.
handle_interact_with_event:
	; try to pick up trash, which has no requirements as to what the 
	; protagonist is already holding
	call handleSecondFloorTrashInteraction
	call handleThirdFloorTrashInteraction

	; if protagonist is not holding the plunger, we don't care to check if
	; he's interacting with the toilets
	ld a, (itemHeld)
	cp ITEM_PLUNGER
	jp nz, handle_interact_with_event_done
	
	; he's holding the plunger, so we can now check each toilet
	call handleSecondFloorToiletInteraction
	call handleThirdFloorToiletInteraction
	call handleFourthFloorToiletInteraction
handle_interact_with_event_done:
	ret
	

; See if we're interacting with the second floor trash can	
handleSecondFloorTrashInteraction:
	ld a, (snowPresent)			; if snow is present, garbage cannot be picked
	cp 0						; up, because the game doesn't support 
								; "dropping" garbage out of the inventory
	jp nz, handleSecondFloorTrashInteraction_done
	
	ld a, (manX)
	cp TRASH_SECOND_FLOOR_X_PIXEL+8		; man has to be directly to the right
										; of the trash can
	jp nz, handleSecondFloorTrashInteraction_done
	
	ld a, (manY)
	cp TRASH_SECOND_FLOOR_Y_PIXEL		; man's top Y equals warning's top Y
	jp nz, handleSecondFloorTrashInteraction_done
	
	ld a, (secondFloorTrashFull)		; if it's not active, don't bother
	cp 0
	jp z, handleSecondFloorTrashInteraction_done
	
	ld a, (itemHeld)
	cp ITEM_GARBAGE						; can't pick up garbage if already
										; holding garbage
	jp z, handleSecondFloorTrashInteraction_done
	
	call deactivateSecondFloorTrash
	call pickup_garbage
	call man_start_working
handleSecondFloorTrashInteraction_done:
	ret


; See if we're interacting with the third floor trash can
handleThirdFloorTrashInteraction:
	ld a, (snowPresent)			; if snow is present, garbage cannot be picked
	cp 0						; up, because the game doesn't support 
								; "dropping" garbage out of the inventory
	jp nz, handleThirdFloorTrashInteraction_done
	
	ld a, (manX)
	cp TRASH_THIRD_FLOOR_X_PIXEL+8		; man has to be directly to the right
										; of the trash can
	jp nz, handleThirdFloorTrashInteraction_done
	
	ld a, (manY)
	cp TRASH_THIRD_FLOOR_Y_PIXEL		; man's top Y equals warning's top Y
	jp nz, handleThirdFloorTrashInteraction_done
	
	ld a, (thirdFloorTrashFull)			; if it's not active, don't bother
	cp 0
	jp z, handleThirdFloorTrashInteraction_done
	
	ld a, (itemHeld)
	cp ITEM_GARBAGE						; can't pick up garbage if already
										; holding garbage
	jp z, handleThirdFloorTrashInteraction_done
	
	call deactivateThirdFloorTrash
	call pickup_garbage
	call man_start_working
handleThirdFloorTrashInteraction_done:
	ret


; See if we're interacting with the second floor toilet
handleSecondFloorToiletInteraction:
	ld a, (manX)
	cp TOILET_SECOND_FLOOR_X_PIXEL-16	; man has to be directly to the left
										; of the toilet
	jp nz, handleSecondFloorToiletInteraction_done
	
	ld a, (manY)
	cp TOILET_SECOND_FLOOR_Y_PIXEL		; man's top Y equals warning's top Y
	jp nz, handleSecondFloorToiletInteraction_done
	
	ld a, (secondFloorToiletClogged)		; if it's not active, don't bother
	cp 0
	jp z, handleSecondFloorToiletInteraction_done
	
	call deactivateSecondFloorToilet
	call man_start_working
handleSecondFloorToiletInteraction_done:
	ret
	

; See if we're interacting with the third floor toilet
handleThirdFloorToiletInteraction:
	ld a, (manX)
	cp TOILET_THIRD_FLOOR_X_PIXEL-16	; man has to be directly to the left
										; of the toilet
	jp nz, handleThirdFloorToiletInteraction_done
	
	ld a, (manY)
	cp TOILET_THIRD_FLOOR_Y_PIXEL		; man's top Y equals warning's top Y
	jp nz, handleThirdFloorToiletInteraction_done
	
	ld a, (thirdFloorToiletClogged)		; if it's not active, don't bother
	cp 0
	jp z, handleThirdFloorToiletInteraction_done
	
	call deactivateThirdFloorToilet
	call man_start_working
handleThirdFloorToiletInteraction_done:
	ret


; See if we're interacting with the fourth floor toilet
handleFourthFloorToiletInteraction:
	ld a, (manX)
	cp TOILET_FOURTH_FLOOR_X_PIXEL+8	; man has to be directly to the right
										; of the toilet
	jp nz, handleFourthFloorToiletInteraction_done
	
	ld a, (manY)
	cp TOILET_FOURTH_FLOOR_Y_PIXEL		; man's top Y equals warning's top Y
	jp nz, handleFourthFloorToiletInteraction_done
	
	ld a, (fourthFloorToiletClogged)		; if it's not active, don't bother
	cp 0
	jp z, handleFourthFloorToiletInteraction_done
	
	call deactivateFourthFloorToilet
	call man_start_working
handleFourthFloorToiletInteraction_done:
	ret

	
deactivateSecondFloorTrash:
	ld hl, secondFloorTrashFull
	ld (hl), 0
	; hide warning sign
	ld b, TRASH_SECOND_FLOOR_Y
	ld c, TRASH_SECOND_FLOOR_X
	ld a, INACTIVE_ATTRIBUTES
	ld d, a
	call bitmaps_colours_set
	call play_trash_pickup_sound
	ret
	
deactivateThirdFloorTrash:
	ld hl, thirdFloorTrashFull
	ld (hl), 0
	; hide warning sign
	ld b, TRASH_THIRD_FLOOR_Y
	ld c, TRASH_THIRD_FLOOR_X
	ld a, INACTIVE_ATTRIBUTES
	ld d, a
	call bitmaps_colours_set
	call play_trash_pickup_sound
	ret

deactivateSecondFloorToilet:
	ld hl, secondFloorToiletClogged
	ld (hl), 0
	; hide warning sign
	ld b, TOILET_SECOND_FLOOR_Y
	ld c, TOILET_SECOND_FLOOR_X
	ld a, INACTIVE_ATTRIBUTES
	ld d, a
	call bitmaps_colours_set
	call play_toilet_unclogging_sound
	ret
	
deactivateThirdFloorToilet:
	ld hl, thirdFloorToiletClogged
	ld (hl), 0
	; hide warning sign
	ld b, TOILET_THIRD_FLOOR_Y
	ld c, TOILET_THIRD_FLOOR_X
	ld a, INACTIVE_ATTRIBUTES
	ld d, a
	call bitmaps_colours_set
	call play_toilet_unclogging_sound
	ret
	
deactivateFourthFloorToilet:
	ld hl, fourthFloorToiletClogged
	ld (hl), 0
	; hide warning sign
	ld b, TOILET_FOURTH_FLOOR_Y
	ld c, TOILET_FOURTH_FLOOR_X
	ld a, INACTIVE_ATTRIBUTES
	ld d, a
	call bitmaps_colours_set
	call play_toilet_unclogging_sound
	ret
	

; Called every frame, this function cycles the animation for the warning
; markers above active objects (toilets, trash bins), as well as randomly
; activates new events.
;
update_events:
	; next events frame
	ld hl, eventFrameCounter
	inc (hl)
	ld hl, framesSinceLastEvent
	inc (hl)
	
	call cycle_warning_sign_graphic
	
	; when protagonist is working, we don't generate events, so that we avoid
	; generating an event for the object on which he is currently working
	ld a, (workingAnimationFrameCountdown)
	cp 0							; if protagonist is working
	jp z, update_events_generate	; then generate events
	ret								; otherwise we're done here
update_events_generate:	
	; if enough time has passed since the last time we generated an event
	; then force it now
	ld a, (framesSinceLastEvent)
	cp FORCE_EVENT_GENERATION_FRAME_COUNT
	jp z, update_events_generate_perform	; force event generation now
	
	; we'll attempt to generate events every now and then
	ld a, (eventFrameCounter)
	and GENERATE_EVENT_FRAME_AND_VALUE
	cp GENERATE_EVENT_FRAME_AND_VALUE
	jp nz, handle_event_generation_done	; not yet cooled down
	
	; should we even generate an event?
	call get_next_random
	cp EVENT_GENERATE_CHANCE
	jp nc, handle_event_generation_done	; if random >= EVENT_GENERATE_CHANCE

update_events_generate_perform:	
	; we are generating an event now
	ld hl, framesSinceLastEvent
	ld (hl), 0
	
	; choose a recipient object for the event
	call get_next_random
	cp 51
	jp c, second_floor_trash
	cp 102
	jp c, second_floor_toilet
	cp 153
	jp c, third_floor_trash
	cp 204
	jp c, third_floor_toilet
	jp fourth_floor_toilet
second_floor_trash:
	ld hl, secondFloorTrashFull
	ld (hl), 1
	call animateSecondFloorTrash
	ret
third_floor_trash:
	ld hl, thirdFloorTrashFull
	ld (hl), 1
	call animateThirdFloorTrash
	ret
second_floor_toilet:
	ld hl, secondFloorToiletClogged
	ld (hl), 1
	call animateSecondFloorToilet
	ret
third_floor_toilet:
	ld hl, thirdFloorToiletClogged
	ld (hl), 1
	call animateThirdFloorToilet
	ret
fourth_floor_toilet:
	ld hl, fourthFloorToiletClogged
	ld (hl), 1
	call animateFourthFloorToilet
	ret	
handle_event_generation_done:
	ret


; Called every frame to animate the warning graphic displayed 
; above each object
;
cycle_warning_sign_graphic:
	ld a, (eventFrameCounter)
	and CYCLE_WARNING_GRAPHIC_AND_VALUE
	cp CYCLE_WARNING_GRAPHIC_AND_VALUE
	jp nz, cycle_warning_sign_graphic_done
	; cycle attributes to make warning signs blink
	
	ld a, (currentAttributes)
	cp ACTIVE_ATTRIBUTES1
	jp z, cycle_warning_sign_graphic_frame2
	ld hl, currentAttributes
	ld (hl), ACTIVE_ATTRIBUTES1
	jp cycle_warning_sign_graphic_redraw
cycle_warning_sign_graphic_frame2:
	ld hl, currentAttributes
	ld (hl), ACTIVE_ATTRIBUTES2

cycle_warning_sign_graphic_redraw:
	; and now redraw all objects
	call animateSecondFloorTrash
	call animateThirdFloorTrash
	call animateSecondFloorToilet
	call animateThirdFloorToilet
	call animateFourthFloorToilet
	call animateSnowPile
cycle_warning_sign_graphic_done:
	ret
	

animateSecondFloorTrash:
	ld a, (secondFloorTrashFull)
	cp 0
	jp z, animateSecondFloorTrash_done
	
	; if snow is present, and currentAttributes equals ACTIVE_ATTRIBUTES1,
	; we switch to ACTIVE_ATTRIBUTES1_T
	ld a, (snowPresent)
	cp 0
	jp z, animateSecondFloorTrash_set_current	; snow not present

	ld a, (currentAttributes)
	cp ACTIVE_ATTRIBUTES2
	jp z, animateSecondFloorTrash_set_current	; we don't modify ATTRIBUTES2
	
	; snow is present, and ACTIVE_ATTRIBUTES1 is current, so use alternate
	ld b, TRASH_SECOND_FLOOR_Y
	ld c, TRASH_SECOND_FLOOR_X
	ld a, ACTIVE_ATTRIBUTES1_T
	ld d, a
	call bitmaps_colours_set
	ret
	
animateSecondFloorTrash_set_current:
	; here, A = attributes
	ld b, TRASH_SECOND_FLOOR_Y
	ld c, TRASH_SECOND_FLOOR_X
	ld a, (currentAttributes)
	ld d, a
	call bitmaps_colours_set
animateSecondFloorTrash_done:
	ret
	

animateThirdFloorTrash:
	ld a, (thirdFloorTrashFull)
	cp 0
	jp z, animateThirdFloorTrash_done
	
	; if snow is present, and currentAttributes equals ACTIVE_ATTRIBUTES1,
	; we switch to ACTIVE_ATTRIBUTES1_T
	ld a, (snowPresent)
	cp 0
	jp z, animateThirdFloorTrash_set_current	; snow not present

	ld a, (currentAttributes)
	cp ACTIVE_ATTRIBUTES2
	jp z, animateThirdFloorTrash_set_current	; we don't modify ATTRIBUTES2
	
	; snow is present, and ACTIVE_ATTRIBUTES1 is current, so use alternate
	ld b, TRASH_THIRD_FLOOR_Y
	ld c, TRASH_THIRD_FLOOR_X
	ld a, ACTIVE_ATTRIBUTES1_T
	ld d, a
	call bitmaps_colours_set
	ret
	
animateThirdFloorTrash_set_current:
	; here, A = attributes
	ld b, TRASH_THIRD_FLOOR_Y
	ld c, TRASH_THIRD_FLOOR_X
	ld a, (currentAttributes)
	ld d, a
	call bitmaps_colours_set
animateThirdFloorTrash_done:
	ret
	

animateSecondFloorToilet:
	ld a, (secondFloorToiletClogged)
	cp 0
	jp z, animateSecondFloorToilet_done
	
	ld b, TOILET_SECOND_FLOOR_Y
	ld c, TOILET_SECOND_FLOOR_X
	ld a, (currentAttributes)
	ld d, a
	call bitmaps_colours_set
animateSecondFloorToilet_done:
	ret
	

animateThirdFloorToilet:
	ld a, (thirdFloorToiletClogged)
	cp 0
	jp z, animateThirdFloorToilet_done
	
	ld b, TOILET_THIRD_FLOOR_Y
	ld c, TOILET_THIRD_FLOOR_X
	ld a, (currentAttributes)
	ld d, a
	call bitmaps_colours_set
animateThirdFloorToilet_done:
	ret
	

animateFourthFloorToilet:
	ld a, (fourthFloorToiletClogged)
	cp 0
	jp z, animateFourthFloorToilet_done
	
	ld b, TOILET_FOURTH_FLOOR_Y
	ld c, TOILET_FOURTH_FLOOR_X
	ld a, (currentAttributes)
	ld d, a
	call bitmaps_colours_set
animateFourthFloorToilet_done:
	ret


animateSnowPile:
	ld a, (snowPresent)
	cp 0
	jp z, animateSnowPile_done
	
	ld b, SNOW_WARNING_Y
	ld c, SNOW_WARNING_X
	ld a, (currentAttributes)
	ld d, a
	call bitmaps_colours_set
animateSnowPile_done:
	ret
	
	
; Returns the number of active events.
; Does NOT include the snow pile
;
; Output:
; 		A - active events count
get_active_events_count:
	ld a, (secondFloorToiletClogged)
	ld hl, thirdFloorToiletClogged
	add a, (hl)
	ld hl, fourthFloorToiletClogged
	add a, (hl)
	ld hl, secondFloorTrashFull
	add a, (hl)
	ld hl, thirdFloorTrashFull
	add a, (hl)
	
	; A = sum of all event statuses (1 when active, 0 when not)
	ret


play_toilet_unclogging_sound:
	ld de, 1
	ld bc, 90
	ld a, 8
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 8
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 130
	ld a, 8
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 90
	ld a, 8
	call sound_manager_effects_enqueue
	ret


play_trash_pickup_sound:
	ld de, 1
	ld bc, 150
	ld a, 8
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 8
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 90
	ld a, 8
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 160
	ld a, 8
	call sound_manager_effects_enqueue
	ret

