;------------------------------------------------------------------------------
; This file is part of the ZX Spectrum game 
;       "Husband Chores" by Sebastian Mihai, 2016, http://sebastianmihai.com
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; This file runs the game itself.
; It continues to either the "game lost" screen, or the "game won" screen.
;
;------------------------------------------------------------------------------

	animationFrameCounter db 0

game_begin:
	ld hl, animationFrameCounter
	ld (hl), 0						; set frame counter to 0
	
	call initialize_new_game
	call display_man	; show the protagonist initially, before he moves
	
	;--------------------------------------------------------------------------
	;                  MAIN GAME LOOP (over 2 video frames)
	;
	; Each iteration of the main game loop takes up two video frames.
	; Since rendering happens in only one of the two frames, the game runs
	; at 25fps (PAL is 50fps).
	; This is needed so that spending time on game logic doesn't cause the 
	; rendering of sprites to flicker.
	;--------------------------------------------------------------------------
	
	;--------------------------------------------------------------------------
	; Frame 0 renders the protagonist and then performs some game logic
	;--------------------------------------------------------------------------
frame0:
	halt				; block and wait for interrupt
	
	ld hl, animationFrameCounter	; increment frame counter
	inc (hl)

	call render_man

	ld a, (workingAnimationFrameCountdown)
	cp 0									; if protagonist is working
	jp nz, frame0_after_keyboard_handling	; then ignore keypresses
	
	ld a, (trashOnGroundVisibleCoundown)
	cp 0									; if trash is still on the ground
	jp nz, frame0_after_keyboard_handling	; then ignore keypresses
	
	call keyboard_read_keys
	call handle_movement_keys
	call handle_action_key
frame0_after_keyboard_handling:
	call update_score
	
	call check_victory
	cp 1
	jp z, game_main_victory					; we have a victory!
	
	call check_loss
	cp 1
	jp z, game_main_loss					; we have a loss!
	
	;--------------------------------------------------------------------------
	; Frame 1 is dedicated to game logic
	;--------------------------------------------------------------------------
frame1:
	halt				; block and wait for previous interrupt to finish
	
	ld hl, animationFrameCounter	; increment frame counter
	inc (hl)

	call update_doors	; we're doing this first because it potentially draws
						; to the screen and we want it to not flicker
						; (the earlier we draw after a halt, the better)
						
	call update_events
	call update_snow
	call update_trash
	
	jp frame0			; back to first frame
	;--------------------------------------------------------------------------
	; End of main loop
	;--------------------------------------------------------------------------


game_main_victory:
	call play_game_won_sound
	ld b, 150			; pause for 3 seconds
game_main_victory_loop:
	halt
	djnz game_main_victory_loop
	; pause over
	call game_won_begin
	ret					; return to main
	
game_main_loss:
	call play_game_lost_sound
	ld b, 150			; pause for 3 seconds
game_main_loss_loop:
	halt
	djnz game_main_loss_loop
	; pause over
	call game_lost_begin
	ret					; return to main


; Checks whether a direction key was pressed, and handles it
;	
handle_movement_keys:
	ld a, (keys_v_c_x_z_cs)
	and KEY_X
	jp z, move_man_right		; if Z is pressed, move man left
	ld a, (keys_v_c_x_z_cs)
	and KEY_Z
	jp z, move_man_left			; if X is pressed, move man right
	
	ret							; no key pressed, so we're done
move_man_right:
	ld hl, manIsFacingLeft
	ld (hl), 0					; face protagonist right
	; now move him
	ld a, (manX)
	inc a
	inc a
	ld d, a						; D := manX + 2
	ld a, (manY)
	ld e, a						; E := manY
	call move_man				; move him!
	call get_next_random		; introduce some randomness
	ret							; we're done

move_man_left:
	ld hl, manIsFacingLeft
	ld (hl), 1					; face protagonist left
	; now move him
	ld a, (manX)
	dec a
	dec a
	ld d, a						; D := manX - 2
	ld a, (manY)
	ld e, a						; E := manY
	call move_man				; move him!
	ret							; we're done


; Checks whether the action key was pressed, and handles it
;
handle_action_key:
	ld b, KEY_M
	call check_just_pressed_keys_b_n_m_ss_sp
	cp 0
	jp z, handle_action_key_done

	; action key was just pressed, so we now handle it by telling all handlers
	; that they should check whether they must perform any actions
	call handle_go_through_door
	call handle_pickup_plunger
	call handle_pickup_shovel
	call handle_interact_with_event
	call handle_dump_garbage
	call handle_shovel_snow
	
handle_action_key_done:
	ret								; M has not just become pressed this frame


play_game_won_sound:
	ld de, 10
	ld bc, 200
	ld a, 15
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 4
	call sound_manager_effects_enqueue

	ld de, 10
	ld bc, 120
	ld a, 7
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 4
	call sound_manager_effects_enqueue
	ld de, 10
	ld bc, 120
	ld a, 30
	call sound_manager_effects_enqueue
	
	ret
	
	
play_game_lost_sound:
	ld de, 5
	ld bc, 300
	ld a, 7
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 4
	call sound_manager_effects_enqueue
	ld de, 5
	ld bc, 400
	ld a, 7
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 4
	call sound_manager_effects_enqueue
	ld de, 5
	ld bc, 500
	ld a, 30
	call sound_manager_effects_enqueue
	
	ret
	
