;------------------------------------------------------------------------------
; This file is part of the ZX Spectrum game 
;       "Husband Chores" by Sebastian Mihai, 2016, http://sebastianmihai.com
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; Sets up an interrupt handler.
; This file must be included AFTER all other code and data, since it ORGs 
; code and data at high addresses, starting at 0xFCFC (64764 decimal). 
;
; An explanation of how this works is described in more detail below.
;
;------------------------------------------------------------------------------

	interruptHandlerActive db 0		; when set to 0, the handler does nothing
	frameCounter db 0


; This routine performs the actual work of the interrupt handler
; Any code that is to be invoked at the beginning of a vblank interrupt
; must be added here.
;
interrupt_handler_worker:
	call sound_manager_continue_playing
	ret
	

; Restores normal interrupt handler function
interrupt_activate:
	ld hl, interruptHandlerActive
	ld (hl), 1
	ret

; Puts interrupt handler in "no-op" mode
interrupt_deactivate:
	ld hl, interruptHandlerActive
	ld (hl), 0
	ret
	
	
; Registers the interrupt handler by setting the I registers
; and filling in the interrupt vector table
initialize_interrupt_handler:
	di			; disable interrupts
	
	call interrupt_deactivate

	ld hl, interruptVectorsTable
	ld a, h
	ld i, a		; I := MSB of interruptVectorsTable

	ld a, $FC
	ld (hl), a	; interruptVectorsTable[0] := 0xFC

	ld de, interruptVectorsTable + 1
	ld bc, 256
	ldir		; for i:=1 to 256
				;     interruptVectorsTable[i] := interruptVectorsTable[i - 1]
				; (essentially sets the following 256 bytes to 0xFC)

	im 2		; switch the CPU to interrupt mode 2
	ei			; re-enable interrupts
	ret
	
	
	; the reason why this is at 0xFCFC is that when an interrupt occurs, it 
	; calculates the address of the pointer to a handler (this one) using
	;     - a known byte (which we put in the I register) and 
	;     - one of 256 addresses, depending what the data bus happens 
	;       to hold at the time
	;
	; the strategy is to put 0xFC in I register, and fill up the potential 
	; 257 bytes (since an address takes up two bytes) with the value 0xFC.
	;
	; this guarantees that whatever byte is on the data bus:
	;     - MSB = register I = 0xFE (the MSB of where our interrupt vectors
	;                                table is located)
	;     - LSB = 0 through 255 (a total of 256 possible values)
	;
	; two bytes are read from this address. To cover this, 257 bytes are 
	; needed, since the 256th value spans into a 257th byte.
	; 
	; hence, our interrupt setup routine has to place 0xFC in 257 contiguous
	; bytes, starting at address 0xFE, to guarantee that 0xFCFC is read
	org $FCFC
interrupt_handler:
	push af
	push hl
	push bc
	push de
	push ix
	push iy
	exx
	ex af, af'
	push af		; af'
	push hl		; hl'
	push bc		; bc'
	push de		; de'
	; all registers have now been pushed
	
	ld a, (interruptHandlerActive)
	cp 0
	jp z, interrupt_handler_restore_and_leave	; if not active, leave
	
	ld hl, frameCounter
	inc (hl)
	
	call interrupt_handler_worker	; do the actual work
	
interrupt_handler_restore_and_leave:
	; now restore all registers
	pop de		; de'
	pop bc		; bc'
	pop hl		; hl'
	pop af		; af'
	ex af, af'
	exx
	
	pop iy
	pop ix
	pop de
	pop bc
	pop hl
	pop af
	ei
	reti

	
	; our 257 byte long interrupt vectors table
	org $FE00						; this must be 256-byte aligned (0x__00)
	interruptVectorsTable ds 257
