;------------------------------------------------------------------------------
; This file is part of the ZX Spectrum game 
;       "Husband Chores" by Sebastian Mihai, 2016, http://sebastianmihai.com
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; Routines for rendering and animating the protagonist of the game
;
;------------------------------------------------------------------------------

	include "graphics\graphics_man_walking_left.asm"
	include "graphics\graphics_man_walking_right.asm"
	include "graphics\graphics_man_working_left.asm"
	include "graphics\graphics_man_working_right.asm"

	INITIAL_X equ 190
	INITIAL_Y equ 168
	MAN_HEIGHT equ 16
	MAN_WIDTH equ 16
	WALKING_ANIMATION_FRAME_CYCLE_LENGTH equ 16	; must be a power of 2
												; and at least 4
	WORKING_ANIMATION_FRAME_COUNT equ 20
	
	oldManIsFacingLeft db 0
	manIsFacingLeft db 0
	
	manX db 0
	manY db 0
	oldManX db 0
	oldManY db 0
	
	manRightX db 0			; used to perform collision detection
	manBottomY db 0
	
	forceRender db 0		; set when we want to re-display the protagonist
							; despite him not having moved
							; one such case is when a door is re-drawn on top 
							; of him, we want to force his re-drawing
	
	lastDisplayedFrameCount db 0 ; stores the value of the frame counter when
								 ; we last displayed our protagonist
								 ; it is used to redraw him when background
								 ; changes underneath him
								 
	workingAnimationFrameCountdown db 0

initialize_man:
	ld hl, oldManIsFacingLeft
	ld (hl), 1
	ld hl, manIsFacingLeft
	ld (hl), 1
	
	ld hl, oldManX
	ld (hl), INITIAL_X
	ld hl, manX
	ld (hl), INITIAL_X
	
	ld hl, oldManY
	ld (hl), INITIAL_Y
	ld hl, manY
	ld (hl), INITIAL_Y
	
	ld hl, lastDisplayedFrameCount
	ld (hl), 0
	
	ld hl, forceRender
	ld (hl), 0
	
	ld hl, workingAnimationFrameCountdown
	ld (hl), 0
	
	call recalculate_after_movement	
	ret


; Recalculates various properties after the protagonist moved.
;
recalculate_after_movement:
	; re-calculate collision box
	ld a, (manX)
	add a, MAN_WIDTH-1
	ld (manRightX), a
	
	ld a, (manY)
	add a, MAN_HEIGHT-1
	ld (manBottomY), a
	ret
	

; Attempt to move man to a new location by a specified amount.
;
; Input:
; 		D - new X position
;       E - new Y position
move_man:
	ld a, (manX)
	ld b, a
	ld a, (manY)
	ld c, a				; BC := (manX)(manY)
	push bc
	
	; update man location
	ld a, d
	ld (manX), a
	ld a, e
	ld (manY), a
	call recalculate_after_movement
	
	; see if it's a valid location to have moved to
	call is_man_obstructed
	cp 0
	jp z, move_man_done	; if not blocked, we're done
	
	; the man is blocked, so in an invalid location
	; the movement is invalid, so we must revert it
	pop bc				; BC := (manX)(manY)
	ld a, b
	ld (manX), a
	ld a, c
	ld (manY), a
	call recalculate_after_movement
	ret

move_man_done:
	pop bc
	ret


; Schedules a forced re-display (no frame advancement) of the protagonist
;
schedule_man_display:
	ld hl, forceRender
	ld (hl), 1
	ret
	

; Determines whether input rectangle overlaps with our protagonist
;
; Input:	
; 		B - rectangle left
;       C - rectangle top
;       D - rectangle right
;       E - rectangle bottom
; Output:
; 		A - 1 if rectangle overlaps the protagonist, 0 otherwise
rectangle_overlap_with_man_test:
	ld a, (manX)
	ld h, a
	ld a, (manY)
	ld l, a
	ld a, (manRightX)
	ld ixh, a
	ld a, (manBottomY)
	ld ixl, a
	
	call rectangle_overlap_test		; A := 1 if man collides with door
	ret
	

; Renders the protagonist (if necessary), erasing him at his old position
; and potentially advancing the animation frame, based on the current 
; frame counter
;
render_man:
	; first determine if anything has changed
	ld a, (oldManX)
	ld hl, manX
	cp (hl)
	jp nz, render_man_perform	; if oldX != currentX then render
	
	ld a, (oldManY)
	ld hl, manY
	cp (hl)
	jp nz, render_man_perform	; else if oldY != currentY then render
	
	ld a, (oldManIsFacingLeft)
	ld hl, manIsFacingLeft
	cp (hl)
	jp nz, render_man_perform	; else if facing changed then render
	
	ld a, (workingAnimationFrameCountdown)
	cp 0
	jp nz, render_man_perform	; else if he is currently working then render
	
	ld a, (forceRender)
	cp 0
	jp nz, render_man_perform_without_animating	; else if forced then render
	
	; nothing changed, so do nothing
	ret
render_man_perform_without_animating:
	ld hl, forceRender
	ld (hl), 0						; clear "force render" flag
	
	call erase_man
	call display_man
	ret
render_man_perform:
	ld hl, forceRender
	ld (hl), 0						; clear "force render" flag

	call erase_man
	ld a, (animationFrameCounter)
	ld hl, lastDisplayedFrameCount
	ld (hl), a						; save frame counter's current value
	call display_man
	call update_man_data
	ret
	

; Updates the old coordinates, facing, state, etc.
;
update_man_data:
	ld a, (manX)
	ld hl, oldManX
	ld (hl), a		; oldX := X
	
	ld a, (manY)
	ld hl, oldManY
	ld (hl), a		; oldY := Y
	
	ld a, (manIsFacingLeft)
	ld hl, oldManIsFacingLeft
	ld (hl), a		; old facing := facing
	
	ret
	

man_start_working:
	ld hl, workingAnimationFrameCountdown
	ld (hl), WORKING_ANIMATION_FRAME_COUNT
	ret
	
	
; Erase our protagonist at his current location by copying the corresponding
; rectangle from the background buffer to the video ram
;
erase_man:
	ld a, (oldManY)
	ld c, a
	ld a, (oldManX)
	ld ixl, a
	ld a, MAN_HEIGHT
	call copy_buffer_rectangle_to_vram ; "erase" first vertical half
	
	ld a, (oldManY)
	ld c, a
	ld a, (oldManX)
	add a, MAN_WIDTH/2	; shift X position by half width to draw second bitmap
	ld ixl, a
	ld a, MAN_HEIGHT
	call copy_buffer_rectangle_to_vram ; "erase" second vertical half
	
	ret


; Displays our protagonist at his current location.
;
display_man:
	;ld a, (manIsWalking)
	;or a							; is the man walking?
	;jp nz, display_man_walking		; yes
	
	ld a, (workingAnimationFrameCountdown)
	cp 0							; is he working?
	jp z, display_man_walking		; no
	
display_man_working:				; yes
	ld a, (workingAnimationFrameCountdown)
	dec a
	ld (workingAnimationFrameCountdown), a	; countdown--
	cp 0
	jp nz, display_man_working_perform	; if we didn't just finish working,
										; don't force a re-rendering
	
	call schedule_man_display		; when the countdown has just reached 0,
									; we just finished working, and must
									; re-displaying the man
display_man_working_perform:
	ld a, (manIsFacingLeft)
	or a							; is the man walking left?
	jp nz, display_man_working_left	; yes
display_man_working_right:			; no
	ld a, (lastDisplayedFrameCount)
	bit 3, a		; animate every few frames
	jp z, display_man_working_right_frame1
display_man_working_right_frame0:
	ld hl, manData0WorkingRightFrame0
	ld de, manData0WorkingRightFrame0Mask
	call display_man_frame
	ret	
display_man_working_right_frame1:
	ld hl, manData0WorkingRightFrame1
	ld de, manData0WorkingRightFrame1Mask
	call display_man_frame
	ret
	
display_man_working_left:
	ld a, (lastDisplayedFrameCount)
	bit 3, a		; animate every few frames
	jp z, display_man_working_left_frame1
display_man_working_left_frame0:
	ld hl, manData0WorkingLeftFrame0
	ld de, manData0WorkingLeftFrame0Mask
	call display_man_frame
	ret	
display_man_working_left_frame1:
	ld hl, manData0WorkingLeftFrame1
	ld de, manData0WorkingLeftFrame1Mask
	call display_man_frame
	ret


display_man_walking:
	ld a, (manIsFacingLeft)
	or a							; is the man walking left?
	jp nz, display_man_walking_left	; yes
	
display_man_walking_right:			; no
	ld a, (lastDisplayedFrameCount)
	and WALKING_ANIMATION_FRAME_CYCLE_LENGTH - 1
	
	cp WALKING_ANIMATION_FRAME_CYCLE_LENGTH * 1 / 4
	jp c, display_man_walking_right_frame1	; if A < 16, frame 1
	cp WALKING_ANIMATION_FRAME_CYCLE_LENGTH * 2 / 4
	jp c, display_man_walking_right_frame2	; if A < 32, frame 2
	cp WALKING_ANIMATION_FRAME_CYCLE_LENGTH * 3 / 4
	jp c, display_man_walking_right_frame3	; if A < 48, frame 3
	
											; else frame 0
display_man_walking_right_frame0:
	ld hl, manData0WalkingRightFrame0
	ld de, manData0WalkingRightFrame0Mask
	call display_man_frame
	ret
display_man_walking_right_frame1:
	ld hl, manData0WalkingRightFrame1
	ld de, manData0WalkingRightFrame1Mask
	call display_man_frame
	ret
display_man_walking_right_frame2:
	ld hl, manData0WalkingRightFrame2
	ld de, manData0WalkingRightFrame2Mask
	call display_man_frame
	ret
display_man_walking_right_frame3:
	ld hl, manData0WalkingRightFrame1
	ld de, manData0WalkingRightFrame1Mask
	call display_man_frame
	ret
	
display_man_walking_left:
	ld a, (lastDisplayedFrameCount)
	and WALKING_ANIMATION_FRAME_CYCLE_LENGTH - 1
	
	cp WALKING_ANIMATION_FRAME_CYCLE_LENGTH * 1 / 4
	jp c, display_man_walking_left_frame1	; if A < 16, frame 1
	cp WALKING_ANIMATION_FRAME_CYCLE_LENGTH * 2 / 4
	jp c, display_man_walking_left_frame2	; if A < 32, frame 2
	cp WALKING_ANIMATION_FRAME_CYCLE_LENGTH * 3 / 4
	jp c, display_man_walking_left_frame3	; if A < 48, frame 3
	
											; else frame 0
display_man_walking_left_frame0:
	ld hl, manData0WalkingLeftFrame0
	ld de, manData0WalkingLeftFrame0Mask
	call display_man_frame
	ret
display_man_walking_left_frame1:
	ld hl, manData0WalkingLeftFrame1
	ld de, manData0WalkingLeftFrame1Mask
	call display_man_frame
	ret
display_man_walking_left_frame2:
	ld hl, manData0WalkingLeftFrame2
	ld de, manData0WalkingLeftFrame2Mask
	call display_man_frame
	ret
display_man_walking_left_frame3:
	ld hl, manData0WalkingLeftFrame1
	ld de, manData0WalkingLeftFrame1Mask
	call display_man_frame
	ret


; Displays our protagonist by rendering two 8x16 bitmaps side by side.
; It assumes that the two data bitmaps are contiguous in memory.
; It assumes that the two mask bitmaps are contiguous in memory.
;
; Input:	
;      HL - pointer to first of two 8x16 data bitmap
;      DE - pointer to first of two 8x16 mask bitmap
display_man_frame:
	push hl
	push de
	
	ld a, (manX)
	ld ixl, a
	ld a, (manY)
	ld c, a
	ld a, MAN_HEIGHT
	call draw_bitmap_OR_blitted_masked_vram	; draw first vertical half
	
	ld b, 0
	ld c, MAN_HEIGHT	; BC := MAN_HEIGHT
	
	pop hl				; HL := DE
	add hl, bc			; HL := HL + MAN_HEIGHT
	ex de, hl			; DE := HL, offsetting DE to second mask bitmap
	pop hl
	add hl, bc			; HL := HL + MAN_HEIGHT, offsetting to second bitmap
	
	ld a, (manX)
	add a, MAN_WIDTH/2	; shift X position by half width to draw second bitmap
	ld ixl, a
	ld a, (manY)
	ld c, a
	ld a, MAN_HEIGHT
	call draw_bitmap_OR_blitted_masked_vram	; draw second vertical half
	
	ret
