;------------------------------------------------------------------------------
; This file is part of the ZX Spectrum game 
;       "Husband Chores" by Sebastian Mihai, 2016, http://sebastianmihai.com
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; Keeps track of the score, based on what's happening in game
;
;------------------------------------------------------------------------------

	SCORE_DELTA_0_EVENTS equ 2
	SCORE_DELTA_1_EVENT equ 0
	SCORE_DELTA_2_EVENTS equ -1
	SCORE_DELTA_3_EVENTS equ -2
	SCORE_DELTA_4_EVENTS equ -3
	SCORE_DELTA_5_EVENTS equ -3

	SCORE_CHECK_FRAME_AND_VALUE equ 31	; how often we check active
										; events to update the score
										; must be a power of 2, minus 1

	SCORE_GAUGE_LEFT equ 240
	SCORE_GAUGE_TOP equ 72
	SCORE_GAUGE_RIGHT equ 247
	SCORE_GAUGE_BOTTOM equ 119
	
	SCORE_GAUGE_HEIGHT equ SCORE_GAUGE_BOTTOM - SCORE_GAUGE_TOP + 1
	SCORE_POINTS_PER_PIXEL equ 4
	MAX_SCORE equ SCORE_GAUGE_HEIGHT * SCORE_POINTS_PER_PIXEL - 1
	MIN_SCORE equ SCORE_POINTS_PER_PIXEL
	
	STARTING_SCORE equ (MAX_SCORE - MIN_SCORE) / 2
										; start with half the progress bar
	
	bitmapTallRectangle ds SCORE_GAUGE_HEIGHT, 255	; one full 8-pixel line
													; for each line in the
													; gauge
	bitmapTallRectangleMask ds SCORE_GAUGE_HEIGHT, 0 ; and the mask, all empty
													 ; 8-pixel lines
	scoreFrameCounter db 0
	score db 0

initialize_score:
	ld hl, score
	ld (hl), STARTING_SCORE
	
	ld hl, scoreFrameCounter
	ld (hl), 0
	
	call display_score
	ret
	

; Fills an empty gauge to display the current score value
;
; MIN_SCORE will display a 1-pixel tall line at the bottom of the gauge
; MIN_SCORE-1 or lower will display an empty gauge, preparing for "you lost!"
; MAX_SCORE will display a full gauge missing one 1-pixel tall line
; MAX_SCORE+1 or higher will display a full gauge, preparing for "you won!"
;
; current score will never be lower than MIN_SCORE-4
;      (since the most it can drop before game over is with 5 active events,
;       causing it to drop to MIN_SCORE - 5 + 1)
;
; current score will never be higher than MAX_SCORE+1
;      (since the most it can increase before winning the game is by 1)
;
display_score:
	; calculate length of gauge in pixels
	ld a, (score)
	cp MIN_SCORE
	jp c, display_score_entirely_empty	; if score < MIN_SCORE then it's empty
	
	ld c, a
	srl c
	srl c			; C : score / 4
					; in other words, C is height of filled portion in pixels
	
	ld a, SCORE_GAUGE_BOTTOM
	sub c			; offset upwards from the bottom of the gauge, as many
					; pixels as the height of the filled portion of the gauge
	inc a			; 0 vs 1 based
	
	push af			; here, A = coordinate of bottom of "empty" area
					
	; now draw it
	ld d, c			; D := height  (temp)
	
	; draw "filled" bottom of gauge
	ld ixl, SCORE_GAUGE_LEFT
	ld c, a			; C := Y coordinate
	ld a, d			; A := height
	ld hl, bitmapTallRectangle		; pointer to bitmap
	call draw_bitmap_OR_blitted_vram
	
	pop af				; A := Y coordinate of bottom of "empty" area
	sub SCORE_GAUGE_TOP	; A := height of "empty" area
	cp 0
	jp z, display_score_done ; it's full, so there's no empty area

	; draw "empty" top of gauge
	ld ixl, SCORE_GAUGE_LEFT
	ld c, SCORE_GAUGE_TOP
	ld hl, bitmapTallRectangle		; pointer to bitmap
	call draw_bitmap_NOT_AND_blitted_vram
	jp display_score_done

display_score_entirely_empty:
	ld ixl, SCORE_GAUGE_LEFT
	ld c, SCORE_GAUGE_TOP
	ld a, SCORE_GAUGE_HEIGHT
	ld hl, bitmapTallRectangle			  ; pointer to bitmap
	call draw_bitmap_NOT_AND_blitted_vram ; draw in erase mode, erasing gauge
	
display_score_done:
	ret
	

; Called every frame, it modifies the score by taking into account
; the number of active events
;
update_score:
	; next frame
	ld hl, scoreFrameCounter
	inc (hl)
	
	; we only check active events to update score every few frames
	ld a, (scoreFrameCounter)
	and SCORE_CHECK_FRAME_AND_VALUE
	cp SCORE_CHECK_FRAME_AND_VALUE
	jp nz, update_score_done	; not yet cooled down

	; update score
	
	call get_active_events_count
	cp 5
	jp z, update_score_5_active
	cp 4
	jp z, update_score_4_active
	cp 3
	jp z, update_score_3_active
	cp 2
	jp z, update_score_2_active
	cp 1
	jp z, update_score_1_active

update_score_0_active:
	ld a, (score)
	add a, SCORE_DELTA_0_EVENTS
	ld (score), a
	jp update_score_display
update_score_5_active:
	ld a, (score)
	add a, SCORE_DELTA_5_EVENTS
	ld (score), a
	jp update_score_display
update_score_4_active:
	ld a, (score)
	add a, SCORE_DELTA_4_EVENTS
	ld (score), a
	jp update_score_display
update_score_3_active:
	ld a, (score)
	add a, SCORE_DELTA_3_EVENTS
	ld (score), a
	jp update_score_display
update_score_2_active:
	ld a, (score)
	add a, SCORE_DELTA_2_EVENTS
	ld (score), a
	jp update_score_display
update_score_1_active:
	ld a, (score)
	add a, SCORE_DELTA_1_EVENT
	ld (score), a
update_score_display:
	call display_score
update_score_done:
	ret


; Check whether the protagonist has won the game
;
; Output:
; 		A - 1 if the game has been won, 0 otherwise
check_victory:
	ld a, MAX_SCORE
	ld hl, score
	cp (hl)
	jp c, check_victory_yes	; if MAX_SCORE - score < 0
	ld a, 0
	ret
check_victory_yes:
	ld a, 1
	ret
	

; Check whether the protagonist has lost the game
;
; Output:
; 		A - 1 if the game has been lost, 0 otherwise
check_loss:
	ld a, (score)
	cp MIN_SCORE
	jp c, check_loss_yes	; if score - MIN_SCORE < 0
	ld a, 0
	ret
check_loss_yes:
	ld a, 1
	ret

