;------------------------------------------------------------------------------
; This file is part of the ZX Spectrum game 
;       "Husband Chores" by Sebastian Mihai, 2016, http://sebastianmihai.com
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; Manages the snow plow arriving and packing snow in the driveway, as well
; as the snow being cleared by the protagonist.
;
;------------------------------------------------------------------------------

	include "graphics\graphics_snow.asm"

	SNOW_CHANCE equ 12						; die roll chance out of 255
	SNOW_ADD_FRAME_CHECK_AND_CP equ 63		; must be a power of 2, minus one
	
	; in 8x8 squares
	SNOW_WARNING_X equ 26
	SNOW_WARNING_Y equ 20
	
	SNOW_X equ 200
	SNOW_Y equ 176
	ADD_SNOW_X_THRESHOLD equ SNOW_X - 24	; protagonist must be to the left
											; of this value in order for snow
											; to be added
	SNOW_PLOW_X equ SNOW_X + 16
	SNOW_PLOW_Y equ SNOW_Y - 16
	SNOW_PLOW_VISIBLE_FRAME_COUNT equ 70

	snowPlowVisibleCoundown db 0
	snowPresent db 0
	snowFrameCounter db 0

	
initialize_snow:
	ld hl, snowPresent
	ld (hl), 0
	
	ld hl, snowFrameCounter
	ld (hl), 0
	
	ret
	

; Called every frame, it takes care of snow-related logic, such as adding
; snow, and hiding the snow plow
;
update_snow:
	ld hl, snowFrameCounter
	inc (hl)
	
	call update_snow_plow
	call snow_try_add
update_snow_done:
	ret


; Tries to add snow
;
snow_try_add:
	ld a, (snowPresent)
	cp 0
	jp nz, snow_try_add_done	; if snow is already present, we're done

	; protagonist has to be to the left of where the snow will appear, so that
	; he doesn't get "trapped" outside the house, without a way to reach the 
	; shovel, which is inside the house
	ld a, ADD_SNOW_X_THRESHOLD
	ld hl, manX
	cp (hl)
	jp c, snow_try_add_done		; if protagonist is not to the left enough 
								; then we won't add snow
	
	ld a, (itemHeld)			; if protagonist is carrying a garbage bag,
	cp ITEM_GARBAGE				; we won't add snow, because we want him to
	jp z, snow_try_add_done		; be able to drop off the garbage outside
	
	ld a, (itemHeld)			; if protagonist is carrying a snow shovel,
	cp ITEM_SHOVEL				; we won't add snow, because we want the snow
	jp z, snow_try_add_done		; plow to have enough time to disappear 
								; before the snow is cleared

	; only attempt to add snow every few frames
	ld a, (snowFrameCounter)
	and SNOW_ADD_FRAME_CHECK_AND_CP
	cp SNOW_ADD_FRAME_CHECK_AND_CP
	jp nz, snow_try_add_done
	
	call get_next_random
	cp SNOW_CHANCE
	jp nc, snow_try_add_done	; if random >= SNOW_CHANCE
								; then snow is not showing up
	
	; we are adding the snow right now
	ld hl, snowPresent
	ld (hl), 1					; mark snow as added
	
	; start countdown timer, which tells us how long to keep the
	; snow plow visible
	ld hl, snowPlowVisibleCoundown
	ld (hl), SNOW_PLOW_VISIBLE_FRAME_COUNT
	
	call display_snow
	call display_snow_plow
	
	call sound_manager_effects_clear	; we want the snow plow sound to
										; interrupt whatever was playing
	call play_snow_plow_arrival_sound
	call sound_manager_effects_lock		; we don't want any sounds from the
										; protagonist interrupting our snow
										; plow beeping
snow_try_add_done:
	ret
	

; Makes snow pile visible
;
display_snow:
	ld hl, bitmapSnowLeft
	ld de, bitmapFilled8x8Square
	ld ixl, SNOW_X
	ld c, SNOW_Y
	ld a, 8
	call draw_bitmap_OR_blitted_masked_vram
	ld hl, bitmapSnowRight
	ld de, bitmapFilled8x8Square
	ld ixl, SNOW_X + 8
	ld c, SNOW_Y
	ld a, 8
	call draw_bitmap_OR_blitted_masked_vram
	ret


; Hides snow pile
;
hide_snow:
	ld ixl, SNOW_X
	ld c, SNOW_Y
	call erase_8x8_bitmap_vram
	ld ixl, SNOW_X + 8
	ld c, SNOW_Y
	call erase_8x8_bitmap_vram
	
	; hide warning indicator
	ld b, SNOW_WARNING_Y
	ld c, SNOW_WARNING_X
	ld a, INACTIVE_ATTRIBUTES
	ld d, a
	call bitmaps_colours_set
	ret	


; Makes snow plow visible
;
display_snow_plow:
	; draw left-most vertical slice
	ld hl, bitmapSnowPlowLeft
	ld de, bitmapFilled8x8Square_Times3
	ld ixl, SNOW_PLOW_X
	ld c, SNOW_PLOW_Y
	ld a, 24
	call draw_bitmap_OR_blitted_masked_vram
	
	; draw middle vertical slice
	ld hl, bitmapSnowPlowMiddle
	ld de, bitmapFilled8x8Square_Times3
	ld ixl, SNOW_PLOW_X + 8
	ld c, SNOW_PLOW_Y
	ld a, 24
	call draw_bitmap_OR_blitted_masked_vram
	
	; draw right-most vertical slice
	ld hl, bitmapSnowPlowRight
	ld de, bitmapFilled8x8Square_Times3
	ld ixl, SNOW_PLOW_X + 16
	ld c, SNOW_PLOW_Y
	ld a, 24
	call draw_bitmap_OR_blitted_masked_vram
	
	; turn flash on for snow plow's light
	ld b, ( SNOW_PLOW_Y - 8 ) / 8		; convert to 8x8 character coordinate
	ld c, ( SNOW_PLOW_X + 8 ) / 8		; convert to 8x8 character coordinate
	ld d, FLASH_ON | BRIGHT_ON | PAPER_BLACK | INK_BLUE
	call bitmaps_colours_set
	ret


; Hides snow plow
;
hide_snow_plow:
	; erase left-most vertical slice
	ld hl, bitmapEmpty8x8Square_Times3
	ld de, bitmapFilled8x8Square_Times3
	ld ixl, SNOW_PLOW_X
	ld c, SNOW_PLOW_Y
	ld a, 24
	call draw_bitmap_OR_blitted_masked_vram
	
	; erase middle vertical slice
	ld hl, bitmapEmpty8x8Square_Times3
	ld de, bitmapFilled8x8Square_Times3
	ld ixl, SNOW_PLOW_X + 8
	ld c, SNOW_PLOW_Y
	ld a, 24
	call draw_bitmap_OR_blitted_masked_vram
	
	; erase right-most vertical slice
	ld hl, bitmapEmpty8x8Square_Times3
	ld de, bitmapFilled8x8Square_Times3
	ld ixl, SNOW_PLOW_X + 16
	ld c, SNOW_PLOW_Y
	ld a, 24
	call draw_bitmap_OR_blitted_masked_vram
	
	; turn flash off for snow plow's light
	ld b, ( SNOW_PLOW_Y - 8 ) / 8		; convert to 8x8 character coordinate
	ld c, ( SNOW_PLOW_X + 8 ) / 8		; convert to 8x8 character coordinate
	ld d, FLASH_OFF | BRIGHT_ON | PAPER_BLACK | INK_BLUE
	call bitmaps_colours_set
	ret
	

; Keeps the snow plow displayed until it has to go away
;
update_snow_plow:
	ld a, (snowPlowVisibleCoundown)
	cp 0
	jp z, update_snow_plow_done		; if plow is not visible, we're done
	
	dec a
	ld (snowPlowVisibleCoundown), a	; countdown--
	
	cp 0
	jp nz, update_snow_plow_done	; if snowPlowVisibleCoundown != 0
	
	; we just reached 0, so hide snow plow
	call hide_snow_plow
	call sound_manager_effects_unlock	; re-allow sounds from the protagonist
										; into the queue
update_snow_plow_done:
	ret


; Called when the player has pressed the action button, it checks 
; whether the player shovels the snow, handling that case, if necessary.
; 
; Snow is never added when the protagonist is already holding the shovel, in 
; order to force him to take at least several seconds from the moment he picks
; up the shovel, until when he reaches the snow pile. By that time, the snow
; plow will have gone away.
;
handle_shovel_snow:
	ld a, (snowPresent)
	cp 0
	jp z, handle_shovel_snow_done		; no snow, so we do nothing
	
	ld a, (manX)
	cp SNOW_X - 16						; man has to be directly to the left
										; of the snow pile
	jp nz, handle_shovel_snow_done
	
	ld a, (manY)
	cp SNOW_Y - 8
	jp nz, handle_shovel_snow_done
	
	ld a, (itemHeld)
	cp ITEM_SHOVEL
	jp nz, handle_shovel_snow_done		; man must be holding shovel

	; shovel!
	
	ld hl, snowPresent					; remove snow
	ld (hl), 0
	
	call hide_snow
	call man_start_working
	call play_shovelling_sound
handle_shovel_snow_done:
	ret


play_shovelling_sound:
	ld de, 1
	ld bc, 130
	ld a, 7
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 5
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 130
	ld a, 7
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 5
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 130
	ld a, 7
	call sound_manager_effects_enqueue
	ret
	
	
play_snow_plow_arrival_sound:
	ld de, 1
	ld bc, 20
	ld a, 25
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 10
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 20
	ld a, 25
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 10
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 20
	ld a, 25
	call sound_manager_effects_enqueue
	ld de, 0
	ld bc, 0
	ld a, 10
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 20
	ld a, 25
	call sound_manager_effects_enqueue
	ret
	
	