;------------------------------------------------------------------------------
; This file is part of the ZX Spectrum game 
;       "Husband Chores" by Sebastian Mihai, 2016, http://sebastianmihai.com
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;
; Manages the trash that is accidentally dropped on the ground by the 
; protagonist on the way to the dumpster.
;
;------------------------------------------------------------------------------

	MAN_DROP_POINT_X equ 208			; protagonist's position when his
	MAN_DROP_POINT_Y equ 168			; garbage may drop on the ground
	
	TRASH_DROP_CHANCE equ 48			; the die we roll to see if we're
										; dropping the garbage
										; out of 255
	
	; we keep trash on the ground for as long as the protagonist 
	; takes to work
	TRASH_ON_GROUND_VISIBLE_FRAME_COUNT equ WORKING_ANIMATION_FRAME_COUNT

	bitmapTrashOnGroundLeft
			db %00000000
			db %00000000
			db %00000000
			db %00010000
			db %00111010
			db %01101101
			db %11011011
			db %11001101

	trashOnGroundVisibleCoundown db 0
	trashFrameCounter db 0
	hasDroppedTrashAlready db 0	; used to prevent multiple snow drops


initialize_trash:
	ld hl, trashOnGroundVisibleCoundown
	ld (hl), 0
	
	ld hl, trashFrameCounter
	ld (hl), 0
	
	ld hl, hasDroppedTrashAlready
	ld (hl), 0
		
	ret
	

; Called every frame, it takes care of trash dropped on the floor.
;
update_trash:
	ld hl, trashFrameCounter
	inc (hl)

	call update_trash_on_ground
	call trash_try_drop
update_trash_done:
	ret


; Tries to drop the protagonist's trash on the ground
;
trash_try_drop:
	ld a, (trashOnGroundVisibleCoundown)
	cp 0
	jp nz, trash_try_drop_done	; if trash is already on ground, we're done

	ld a, (itemHeld)			; if protagonist is not carrying a garbage bag
	cp ITEM_GARBAGE				; we're done
	jp nz, trash_try_drop_done
	
	ld a, MAN_DROP_POINT_X
	ld hl, manX
	cp (hl)
	jp nz, trash_try_drop_done	; if manX != dropX, we're done
	
	ld a, MAN_DROP_POINT_Y
	ld hl, manY
	cp (hl)
	jp nz, trash_try_drop_done	; if manY != dropY, we're done
	
	ld a, (hasDroppedTrashAlready)
	cp 0
	jp nz, trash_try_drop_done	; don't drop the same trash again
	
	call get_next_random
	cp TRASH_DROP_CHANCE
	jp nc, trash_try_drop_done	; if random >= TRASH_DROP_CHANCE
								; then we won't drop the trash
	
	; drop man's trash to the ground!
	ld hl, trashOnGroundVisibleCoundown
	ld (hl), TRASH_ON_GROUND_VISIBLE_FRAME_COUNT	; start counter
	
	ld hl, hasDroppedTrashAlready
	ld (hl), 1
	
	call pickup_clear				; clear inventory
	call display_trash_on_ground	; show trash on the ground
	call man_start_working
	call play_trash_dropped_sound
trash_try_drop_done:
	ret
	

display_trash_on_ground:
	; IXL must be set according to which direction the protagonist
	; is facing, so that we guarantee trash always drops in front of him
	ld ixl, MAN_DROP_POINT_X + 16		; assume he is facing right
	ld a, (manIsFacingLeft)
	cp 0
	jp z, display_trash_on_ground_perform	; he is indeed facing right
	ld ixl, MAN_DROP_POINT_X - 8		; he is actually facing left
display_trash_on_ground_perform:
	ld hl, bitmapTrashOnGroundLeft
	ld de, bitmapFilled8x8Square
	ld c, MAN_DROP_POINT_Y + 8
	ld a, 8
	call draw_bitmap_OR_blitted_masked_vram
	ret


erase_trash_from_ground:
	; IXL must be set according to which direction the protagonist
	; is facing, so that we guarantee trash always drops in front of him
	ld ixl, MAN_DROP_POINT_X + 16		; assume he is facing right
	ld a, (manIsFacingLeft)
	cp 0
	jp z, erase_trash_from_ground_perform	; he is indeed facing right
	ld ixl, MAN_DROP_POINT_X - 8		; he is actually facing left
erase_trash_from_ground_perform:
	ld hl, bitmapEmpty8x8Square
	ld de, bitmapFilled8x8Square
	ld c, MAN_DROP_POINT_Y + 8
	ld a, 8
	call draw_bitmap_OR_blitted_masked_vram
	ret
	

; Keeps the trash on the ground until it has to go away
;
update_trash_on_ground:
	ld a, (trashOnGroundVisibleCoundown)
	cp 0
	jp z, update_trash_on_ground_done ; if trash is not on ground, we're done
	
	dec a
	ld (trashOnGroundVisibleCoundown), a	; countdown--
	
	cp 0
	jp nz, update_trash_on_ground_done	; if coundown != 0, we continue to 
										; keep trash on the ground
	
	; we just reached 0, so give control back to the protagonist, 
	; clear the trash off the ground, and make him hold a garbage bag again
	call erase_trash_from_ground
	call pickup_garbage
update_trash_on_ground_done:
	ret


play_trash_dropped_sound:
	ld de, 1
	ld bc, 200
	ld a, 7
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 150
	ld a, 7
	call sound_manager_effects_enqueue
	ld de, 1
	ld bc, 100
	ld a, 7
	call sound_manager_effects_enqueue
	ret

